@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) 2010, 2011 Sebastian Wiesner <lunaryorn@googlemail.com>
+# Copyright (C) 2010, 2011, 2012 Sebastian Wiesner <lunaryorn@googlemail.com>
 
 # This library is free software; you can redistribute it and/or modify it
 # under the terms of the GNU Lesser General Public License as published by the
@@ -19,15 +19,64 @@
 from __future__ import (print_function, division, unicode_literals,
                         absolute_import)
 
-import sys, os
+import sys
+import os
 
 from docutils import nodes
 from docutils.parsers.rst import Directive
 
+# add the pyudev source directory to our path
 doc_directory = os.path.dirname(os.path.abspath(__file__))
 sys.path.append(os.path.normpath(
     os.path.join(doc_directory, os.pardir)))
 
+
+class Mock(object):
+    """
+    Mock modules.
+
+    Taken from
+    http://read-the-docs.readthedocs.org/en/latest/faq.html#i-get-import-errors-on-libraries-that-depend-on-c-modules
+    with some slight changes.
+    """
+
+    @classmethod
+    def mock_modules(cls, *modules):
+        for module in modules:
+            sys.modules[module] = cls()
+
+    def __init__(self, *args, **kwargs):
+        pass
+
+    def __call__(self, *args, **kwargs):
+        return self.__class__()
+
+    def __getattr__(self, attribute):
+        if attribute in ('__file__', '__path__'):
+            return os.devnull
+        else:
+            # return the *class* object here.  Mocked attributes may be used as
+            # base class in pyudev code, thus the returned mock object must
+            # behave as class, or else Sphinx autodoc will fail to recognize
+            # the mocked base class as such, and "autoclass" will become
+            # meaningless
+            return self.__class__
+
+
+# mock out native modules used throughout pyudev to enable Sphinx autodoc even
+# if these modules are unavailable, as on readthedocs.org
+Mock.mock_modules('PyQt4', 'PyQt4.QtCore', 'PySide', 'PySide.QtCore',
+                  'glib', 'gobject', 'wx', 'wx.lib', 'wx.lib.newevent',
+                  'pyudev._libudev')
+
+# mock out the NewEvent function of wxPython.  Let's praise the silly wx API
+def NewEventMock():
+    yield 'event_class'
+    yield 'event_constant'
+
+sys.modules['wx.lib.newevent'].NewEvent = NewEventMock
+
+
 import pyudev
 
 needs_sphinx = '1.0'
