@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) 2010, 2011 Sebastian Wiesner <lunaryorn@googlemail.com>
+# Copyright (C) 2010, 2011, 2012 Sebastian Wiesner <lunaryorn@googlemail.com>
 
 # This library is free software; you can redistribute it and/or modify it
 # under the terms of the GNU Lesser General Public License as published by the
@@ -22,7 +22,7 @@
 
     A binding to libudev_.
 
-    .. _libudev: http://www.kernel.org/pub/linux/utils/kernel/hotplug/udev.html
+    .. _libudev: http://www.kernel.org/pub/linux/utils/kernel/hotplug/libudev/
 
     Usage
     -----
@@ -86,6 +86,10 @@
     ...     if action == 'add':
     ...         print('{0!r} added'.format(device))
 
+    To conveniently observe monitors in background, the
+    :class:`MonitorObserver` class is provided, which implements a background
+    thread that continously receives events from a :class:`Monitor`.
+
     Accessing devices directly
     ^^^^^^^^^^^^^^^^^^^^^^^^^^
 
@@ -134,8 +138,8 @@
     Toolkit integration
     ^^^^^^^^^^^^^^^^^^^
 
-    pyudev provides classes, which integrate monitoring into the event loop
-    of GUI toolkits.  Currently, PyQt4, PySide and pygobject are supported.
+    pyudev provides classes, which integrate monitoring into the event loop of
+    GUI toolkits.  Currently, PyQt4, PySide, pygobject and wx are supported.
 
 
     A note on versioning
@@ -169,7 +173,7 @@ from __future__ import (print_function, division, unicode_literals,
                         absolute_import)
 
 
-__version__ = '0.13'
+__version__ = '0.14'
 __version_info__ = tuple(map(int, __version__.split('.')))
 __all__ = ['Context', 'Device']
 
